local L = AceLibrary("AceLocale-2.2"):new("Talented")
local Talented = Talented

local BuildTalentInfo
do
	local GetNumTalentTabs = GetNumTalentTabs
	local GetTalentTabInfo = GetTalentTabInfo
	local GetNumTalents = GetNumTalents
	local GetTalentInfo = GetTalentInfo
	local GetTalentPrereqs = GetTalentPrereqs
	
	local function FillPrereqs(...)
		if select("#", ...) < 3 then return end
		local table = {}
		for i = 1, select("#", ...), 3 do
			local row, column = select(i, ...)
			table[#table + 1] = { row = row, column = column }
		end
		return table
	end
	
	function BuildTalentInfo()
		local result = {
		}
		
		for tab = 1, GetNumTalentTabs() do
			local name, icon, _, background = GetTalentTabInfo(tab)
			result[tab] = {
				info = {
					name = name,
					icon = icon,
					background = background,
				},
				talents = {}
			}
			local talents = result[tab].talents
			for index = 1, GetNumTalents(tab) do
				local name, icon, row, column, rank, ranks, exceptional = GetTalentInfo(tab, index)
				talents[index] = {
					info = {
						name = name,
						icon = icon,
						row = row,
						column = column,
						ranks = ranks,
						exceptional = exceptional,
						prereqs = FillPrereqs(GetTalentPrereqs(tab, index)),
					},
				}
			end
		end
		
		return result
	end
end

local DataAddonLoaded
do
	local loaded
	function DataAddonLoaded()
		if loaded ~= nil then 
			return loaded
		end
		if not IsAddOnLoaded("Talented_Data") then
			LoadAddOn("Talented_Data")
		end
		loaded = Talented_Data and true or false
		if loaded then
			local opt = Talented.options.args.template.args.new.args
			
			opt.empty = {
					name = L["New empty template"],
					desc = L["Create a new template from scratch."],
					type = "group",
					order = 100,
					args = {
					},
			}
			
			for name, info in pairs(Talented_Data) do
				opt.empty.args[name] = {
					name = info.name,
					desc = info.name,
					type = "execute",
					order = 100,
					func = function() Talented:SetTemplate(Talented:CreateEmptyTemplate(name)) end,
				}
			end
		end
		return loaded
	end
end

local function FindTalentIndex(table, row, column)
	for index, talent in ipairs(table) do
		if talent.info.row == row and talent.info.column == column then
			return index
		end
	end
end


local function FindPrereqsSource(info)
	for tab, tree in ipairs(info) do
		for index, talent in ipairs(tree.talents) do
			local p = talent.info.prereqs
			if p then
				for _, req in ipairs(p) do
					if req.source then
						return
					else
						req.source = FindTalentIndex(tree.talents, req.row, req.column)
					end
				end
			end
		end
	end
end

function Talented:GetTalentInfo(class)
	-- class = class or PlayerClass
	if not self.talents then
		if DataAddonLoaded() then
			self.talents = Talented_Data
		else
			self.talents = {
				[select(2, UnitClass("player"))] = BuildTalentInfo(),
			}
		end
	end
	local info = self.talents[class]
	if info then
		FindPrereqsSource(info)
	else
		self:ReportMissingTalents(class)
	end
	return info
end

function Talented:ReportMissingTalents(class)
	if self.db.profile.report_unavailable or not self.db.profile.reported_once then
		self:Print(L["Talented can not perform the required action because it does not have the required talent data available for class %s."]:format(class))
		if not self.db.profile.reported_once then
			self.db.profile.reported_once = true
			self:Print(L["You must install the Talented_Data helper addon for this action to work."])
			self:Print(L["You can download it from http://files.wowace.com/ ."])
		end
	end
end
